@@ -135,6 +135,9 @@
 #include "nsIDOMHTMLIFrameElement.h"
 #include "nsIViewManager.h"
 
+#include "nsITextControlFrame.h"
+#include "nsGUIEvent.h"
+
 #ifdef MOZ_XUL
 #include "nsIDOMXULTextboxElement.h"
 #endif
@@ -813,13 +816,16 @@ MicrobEalContextMenuInfo::InsertTextToNode(nsIDOMNode *aDOMNode, nsAString &aStr
     } else
       buffer.Assign(aString);
 
-    input->SetValue(buffer);
-    int len = aString.Length() + surr1;
-    nsinput->SetSelectionRange(selectionStart + len, selectionStart + len);
-
     nsCOMPtr<nsIDOMNSEditableElement> nseditable;
     nseditable = do_QueryInterface(targetNode, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
+    nseditable->SetUserInput(buffer);
+
+    rv = DoHandleEventWithTarget(aDOMNode);
+    NS_ENSURE_SUCCESS(rv, rv);
+   
+    int len = aString.Length() + surr1;
+    nsinput->SetSelectionRange(selectionStart + len, selectionStart + len);
 
     nsCOMPtr<nsIEditor> theEditor;
     nseditable->GetEditor(getter_AddRefs(theEditor));
@@ -1302,8 +1308,10 @@ MicrobEalContextMenuInfo::GetViewToScroll(nsIDOMNode *aDOMNode, nsIScrollableVie
   nsCOMPtr <nsIContent> content = do_QueryInterface(aDOMNode, &rv);
   NS_ENSURE_TRUE(content, rv);
 
-  nsIDocument* doc = content->GetDocument();
-  NS_ENSURE_TRUE(doc, NS_ERROR_FAILURE);
+  nsCOMPtr<nsIDOMDocument> nodeDoc;
+  rv = aDOMNode->GetOwnerDocument(getter_AddRefs(nodeDoc));
+  nsCOMPtr<nsIDocument> doc = do_QueryInterface(nodeDoc, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsIPresShell *presShell = doc->GetPrimaryShell();
   NS_ENSURE_TRUE(presShell, NS_ERROR_FAILURE);
@@ -1390,6 +1398,43 @@ MicrobEalContextMenuInfo::ScrollScrollableView(PRInt32 aDX, PRInt32 aDY)
   return mScrollableView->ScrollByPixels(-aDX, -aDY, overflowX, overflowY);
 }
 
+nsresult
+MicrobEalContextMenuInfo::DoHandleEventWithTarget(nsIDOMNode *aDOMNode)
+{
+  ULOG();
+
+  nsresult rv;
+
+  nsCOMPtr<nsIContent> content = do_QueryInterface(aDOMNode, &rv);
+  NS_ENSURE_TRUE(content, rv);
+
+  nsCOMPtr<nsIDOMDocument> nodeDoc;
+  rv = aDOMNode->GetOwnerDocument(getter_AddRefs(nodeDoc));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIDocument> doc = do_QueryInterface(nodeDoc, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsIPresShell *presShell = doc->GetPrimaryShell();
+  NS_ENSURE_TRUE(presShell, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIDOMHTMLInputElement> input;
+  input = do_QueryInterface(aDOMNode);
+
+  PRUint32 eventType = NS_FORM_INPUT;
+
+  if (input) {
+    nsString value;
+    input->GetValue(value);
+    if (!value.IsEmpty())
+      eventType = NS_FORM_CHANGE;
+  }
+
+  nsEventStatus status = nsEventStatus_eIgnore;
+  nsInputEvent event(PR_TRUE, eventType, nsnull);
+  return presShell->HandleEventWithTarget(&event, nsnull, content, &status);
+}
+
 nsresult
 MicrobEalContextMenuInfo::UpdateScrollableView(nsIDOMNode *aNode, int aDirection)
 {
